@@ -29,7 +29,7 @@
 #include <time.h>
 
 #include <math.h>
-#include <cairo/cairo.h>
+#include <cairo.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -365,6 +365,9 @@ chooser_preview_update (GtkFileChooser *file_chooser,
 	preview = GTK_WIDGET (data);
 	filename = gtk_file_chooser_get_preview_filename (file_chooser);
 
+	if (filename == NULL)
+		return;
+
 	pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
 	have_preview = (pixbuf != NULL);
 	g_free (filename);
@@ -1583,42 +1586,6 @@ fish_applet_change_orient (PanelApplet       *applet,
 		update_pixmap (fish);
 }
 
-static void
-fish_applet_change_background (PanelApplet               *panel_applet,
-			       PanelAppletBackgroundType  type,
-			       GdkColor                  *color,
-			       GdkPixmap                 *pixmap)
-{
-	FishApplet *applet;
-	GtkRcStyle *rc_style;
-	GtkStyle   *style;
-
-	applet = (FishApplet *) panel_applet;
-
-	/* reset style */
-	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
-	gtk_rc_style_unref (rc_style);
-
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		gtk_widget_modify_bg (GTK_WIDGET (applet),
-				      GTK_STATE_NORMAL, color);
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		style = gtk_style_copy (GTK_WIDGET (applet)->style);
-		if (style->bg_pixmap[GTK_STATE_NORMAL])
-			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
-		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
-		gtk_widget_set_style (GTK_WIDGET (applet), style);
-		g_object_unref (style);
-		break;
-	}
-}
-
 static void
 change_water (FishApplet *fish)
 {
@@ -2050,6 +2017,9 @@ fish_applet_instance_init (FishApplet      *fish,
 
 	panel_applet_set_flags (PANEL_APPLET (fish),
 				PANEL_APPLET_EXPAND_MINOR);
+
+	panel_applet_set_background_widget (PANEL_APPLET (fish),
+					    GTK_WIDGET (fish));
 }
 
 static void
@@ -2061,7 +2031,6 @@ fish_applet_class_init (FishAppletClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	applet_class->change_orient = fish_applet_change_orient;
-	applet_class->change_background = fish_applet_change_background;
 
 	gtkobject_class->destroy = fish_applet_destroy;
 
