@@ -32,6 +32,7 @@
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtktooltips.h>
 #include <libwnck/screen.h>
+#include <libwnck/util.h>
 
 #include "wncklet.h"
 #include "window-menu.h"
@@ -134,37 +135,6 @@ wncklet_display_help (GtkWidget  *widget,
 		g_error_free (error);
 	}
 }
-void
-wncklet_change_background (GtkWidget                 *widget,
-			   PanelAppletBackgroundType  type,
-			   GdkColor                  *color,
-			   GdkPixmap                 *pixmap)
-{
-	GtkRcStyle *rc_style;
-	GtkStyle   *style;
-
-	/* reset style */
-	gtk_widget_set_style (widget, NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (widget, rc_style);
-	gtk_rc_style_unref (rc_style);
-
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, color);
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		style = gtk_style_copy (widget->style);
-		if (style->bg_pixmap[GTK_STATE_NORMAL])
-			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
-		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
-		gtk_widget_set_style (widget, style);
-		g_object_unref (style);
-		break;
-	}
-}
 
 WnckScreen *
 wncklet_get_screen (GtkWidget *applet)
@@ -203,7 +173,13 @@ wncklet_factory (PanelApplet *applet,
 		 gpointer     data)
 {
 	gboolean retval = FALSE;
-	
+	static gboolean type_registered = FALSE;
+
+	if (!type_registered) {
+		wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
+		type_registered = TRUE;
+	}
+
 	if (!strcmp (iid, "OAFIID:GNOME_WindowMenuApplet"))
 		retval = window_menu_applet_fill (applet);
 
