@@ -406,16 +406,16 @@ panel_applet_frame_change_orientation (PanelAppletFrame *frame,
 
 	switch (orientation) {
 	case PANEL_ORIENTATION_TOP:
-		orient = GNOME_Vertigo_PANEL_ORIENT_UP;
-		break;
-	case PANEL_ORIENTATION_BOTTOM:
 		orient = GNOME_Vertigo_PANEL_ORIENT_DOWN;
 		break;
+	case PANEL_ORIENTATION_BOTTOM:
+		orient = GNOME_Vertigo_PANEL_ORIENT_UP;
+		break;
 	case PANEL_ORIENTATION_LEFT:
-		orient = GNOME_Vertigo_PANEL_ORIENT_LEFT;
+		orient = GNOME_Vertigo_PANEL_ORIENT_RIGHT;
 		break;
 	case PANEL_ORIENTATION_RIGHT:
-		orient = GNOME_Vertigo_PANEL_ORIENT_RIGHT;
+		orient = GNOME_Vertigo_PANEL_ORIENT_LEFT;
 		break;
 	default:
 		g_assert_not_reached ();
@@ -491,6 +491,14 @@ panel_applet_frame_change_background (PanelAppletFrame    *frame,
 	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
 	g_return_if_fail (PANEL_IS_WIDGET (GTK_WIDGET (frame)->parent));
 
+	if (frame->priv->has_handle) {
+		PanelBackground *background;
+
+		background = &PANEL_WIDGET (GTK_WIDGET (frame)->parent)->background;
+		panel_background_change_background_on_widget (background,
+							      GTK_WIDGET (frame));
+	}
+
 	bg_str = panel_applet_frame_get_background_string (
 			frame, PANEL_WIDGET (GTK_WIDGET (frame)->parent), type);
 
@@ -1028,7 +1036,9 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 	gtk_widget_show_all (dialog);
 
 	g_free (frame->priv->iid);
-	g_free (frame);
+	frame->priv->iid = NULL;
+
+	gtk_widget_destroy (GTK_WIDGET (frame));
 }
 
 static void
@@ -1127,16 +1137,16 @@ panel_applet_frame_get_orient_string (PanelAppletFrame *frame,
 
 	switch (orientation) {
 	case PANEL_ORIENTATION_TOP:
-		retval = "up";
-		break;
-	case PANEL_ORIENTATION_BOTTOM:
 		retval = "down";
 		break;
+	case PANEL_ORIENTATION_BOTTOM:
+		retval = "up";
+		break;
 	case PANEL_ORIENTATION_LEFT:
-		retval = "left";
+		retval = "right";
 		break;
 	case PANEL_ORIENTATION_RIGHT:
-		retval = "right";
+		retval = "left";
 		break;
 	default:
 		g_assert_not_reached ();
