@@ -39,6 +39,7 @@
 #include <glib/gi18n.h>
 
 #include <libgnomevfs/gnome-vfs.h>
+#include <libgnome/gnome-url.h>
 #include <libgnomeui/gnome-url.h>
 
 #include "menu.h"
@@ -52,6 +53,9 @@
 #define BOOKMARKS_FILENAME      ".gtk-bookmarks"
 #define DESKTOP_IS_HOME_DIR_DIR "/apps/nautilus/preferences"
 #define DESKTOP_IS_HOME_DIR_KEY "/apps/nautilus/preferences/desktop_is_home_dir"
+#define NAMES_DIR               "/apps/nautilus/desktop"
+#define HOME_NAME_KEY           "/apps/nautilus/desktop/home_icon_name"
+#define COMPUTER_NAME_KEY       "/apps/nautilus/desktop/computer_icon_name"
 #define MAX_ITEMS_OR_SUBMENU    5
 
 #define PANEL_PLACE_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_PLACE_MENU_ITEM, PanelPlaceMenuItemPrivate))
@@ -99,23 +103,25 @@ activate_uri (GtkWidget *menuitem,
 						       GNOME_VFS_MAKE_URI_DIR_HOMEDIR);
 	gnome_url_show_on_screen (url, screen, &error);
 
-	if (error) {
-		escaped = g_markup_escape_text (url, -1);
-		panel_error_dialog (screen, "cannot_show_url", TRUE,
-				    _("Cannot display location '%s'"),
-				    "%s",
-				    escaped,
-				    error->message);
-
+	if (error != NULL) {
+		if (error->code != GNOME_URL_ERROR_CANCELLED) {
+			escaped = g_markup_escape_text (url, -1);
+			panel_error_dialog (screen, "cannot_show_url", TRUE,
+					    _("Could not open location '%s'"),
+					    "%s",
+					    escaped,
+					    error->message);
+			g_free (escaped);
+		}
 		g_error_free (error);
-		g_free (escaped);
 	}
 	g_free (url);
 }
  
 static void
 panel_menu_items_append_from_desktop (GtkWidget *menu,
-				      char      *path)
+				      char      *path,
+				      char      *force_name)
 {
 	GKeyFile  *key_file;
 	gboolean   loaded;
@@ -166,11 +172,15 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 
 	icon    = g_key_file_get_locale_string (key_file, "Desktop Entry",
 						"Icon", NULL, NULL);
-	name    = g_key_file_get_locale_string (key_file, "Desktop Entry",
-						"Name", NULL, NULL);
 	comment = g_key_file_get_locale_string (key_file, "Desktop Entry",
 						"Comment", NULL, NULL);
 
+	if (string_empty (force_name))
+		name = g_key_file_get_locale_string (key_file, "Desktop Entry",
+						     "Name", NULL, NULL);
+	else
+		name = g_strdup (force_name);
+
 	item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
 				   icon, NULL, name, TRUE);
@@ -271,14 +281,13 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 	table = g_hash_table_new (g_str_hash, g_str_equal);
 	add_bookmarks = NULL;
 
-	//FIXME gnome_vfs_uri_exists: could there be an authentication
-	//dialog if there's a bookmarks to sftp://something?
 	for (i = 0; lines[i]; i++) {
 		if (lines[i][0] && !g_hash_table_lookup (table, lines[i])) {
 			GnomeVFSURI *uri;
 			char        *space;
 			char        *label;
-			char        *unescaped_uri;
+
+			g_hash_table_insert (table, lines[i], lines[i]);
 
 			space = strchr (lines[i], ' ');
 			if (space) {
@@ -288,15 +297,15 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 				label = NULL;
 			}
 
-			unescaped_uri = gnome_vfs_unescape_string (lines[i],
-								   "");
-			uri = gnome_vfs_uri_new (unescaped_uri);
-			g_free (unescaped_uri);
+			uri = gnome_vfs_uri_new (lines[i]);
 
-			if (!gnome_vfs_uri_exists (uri)) {
+			if (!uri ||
+			    (!strcmp (gnome_vfs_uri_get_scheme (uri), "file") &&
+			     !gnome_vfs_uri_exists (uri))) {
 				if (label)
 					g_free (label);
-				gnome_vfs_uri_unref (uri);
+				if (uri)
+					gnome_vfs_uri_unref (uri);
 				continue;
 			}
 
@@ -304,7 +313,6 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 			bookmark->uri = uri;
 			bookmark->label = label;
 			add_bookmarks = g_slist_prepend (add_bookmarks, bookmark);
-			g_hash_table_insert (table, lines[i], lines[i]);
 		}
 	}
 
@@ -358,13 +366,32 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 			char *buffer;
 
 			if (gnome_vfs_uri_is_local (bookmark->uri)) {
-				buffer = gnome_vfs_uri_get_path (bookmark->uri);
+				buffer = gnome_vfs_get_local_path_from_uri (full_uri);
 				label = g_filename_display_basename (buffer);
 			} else {
-				buffer = gnome_vfs_uri_extract_short_name (bookmark->uri);
-				label = g_filename_display_name (buffer);
-				g_free (buffer);
+				const char *hostname;
+				char       *displayname;
+
+				hostname = gnome_vfs_uri_get_host_name (bookmark->uri);
+				buffer = gnome_vfs_uri_get_path (bookmark->uri);
+				if (!buffer || !buffer[0])
+					displayname = g_strdup ("/");
+				else
+					displayname = g_filename_display_basename (buffer);
+				if (hostname) {
+					/* Translators: the first string is a
+					 * path and the second string is a
+					 * hostname. nautilus contains the same
+					 * string to translate. */
+					label = g_strdup_printf (_("%1$s on %2$s"),
+								 displayname,
+								 hostname);
+					g_free (displayname);
+				} else
+					label = displayname;
 			}
+
+			g_free (buffer);
 		}
 
 		panel_menu_items_append_place_item ("gnome-fs-directory",
@@ -386,6 +413,28 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 	g_slist_free (add_bookmarks);
 }
 
+static gint
+panel_place_menu_item_sort_volume (gconstpointer a,
+				   gconstpointer b) {
+	const GnomeVFSVolume *volume_a = a;
+	const GnomeVFSVolume *volume_b = b;
+	char                 *display_a;
+	char                 *display_b;
+	int                   retval;
+
+	g_return_val_if_fail (a && b, 0);
+
+	display_a = gnome_vfs_volume_get_display_name (volume_a);
+	display_b = gnome_vfs_volume_get_display_name (volume_b);
+
+	retval = g_utf8_collate (display_a, display_b);
+
+	g_free (display_a);
+	g_free (display_b);
+
+	return retval;
+}
+
 static void
 panel_place_menu_item_append_volumes (GtkWidget *menu,
 				      gboolean   connected_volumes)
@@ -418,17 +467,32 @@ panel_place_menu_item_append_volumes (GtkWidget *menu,
 		}
 	}
 
-	add_volumes = g_slist_reverse (add_volumes);
+	if (connected_volumes)
+		add_volumes = g_slist_sort (add_volumes,
+					    panel_place_menu_item_sort_volume);
+	else
+		add_volumes = g_slist_reverse (add_volumes);
 
 	if (g_slist_length (add_volumes) <= MAX_ITEMS_OR_SUBMENU) {
 		add_menu = menu;
 	} else {
-		GtkWidget *item;
+		GtkWidget  *item;
+		const char *title;
+		const char *icon;
 
-		if (connected_volumes)
-			item = gtk_menu_item_new_with_label (_("Network Places"));
-		else
-			item = gtk_menu_item_new_with_label (_("Removable Media"));
+		/* FIXME: find nice icons. We don't use NULL since it will
+		 * make the menu items smaller... */
+		if (connected_volumes) {
+			title = _("Network Places");
+			icon = "";
+		} else {
+			title = _("Removable Media");
+			icon = "";
+		}
+
+		item = gtk_image_menu_item_new ();
+		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
+					   icon, NULL, title, TRUE);
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
@@ -471,11 +535,18 @@ panel_place_menu_item_create_menu (EggRecentViewGtk **recent_view)
 {
 	GtkWidget *places_menu;
 	GtkWidget *item;
+	char      *gconf_name;
 
 	places_menu = panel_create_menu ();
 
+	gconf_name = gconf_client_get_string (panel_gconf_get_client (),
+					      HOME_NAME_KEY,
+					      NULL);
 	panel_menu_items_append_from_desktop (places_menu,
-					      "nautilus-home.desktop");
+					      "nautilus-home.desktop",
+					      gconf_name);
+	if (gconf_name)
+		g_free (gconf_name);
 
 	if (!gconf_client_get_bool (panel_gconf_get_client (),
 				    DESKTOP_IS_HOME_DIR_KEY,
@@ -495,13 +566,25 @@ panel_place_menu_item_create_menu (EggRecentViewGtk **recent_view)
 	panel_place_menu_item_append_gtk_bookmarks (places_menu);
 	add_menu_separator (places_menu);
 
+	gconf_name = gconf_client_get_string (panel_gconf_get_client (),
+					      COMPUTER_NAME_KEY,
+					      NULL);
 	panel_menu_items_append_from_desktop (places_menu,
-					      "nautilus-computer.desktop");
+					      "nautilus-computer.desktop",
+					      gconf_name);
+	if (gconf_name)
+		g_free (gconf_name);
+
+	panel_menu_items_append_from_desktop (places_menu,
+					      "nautilus-cd-burner.desktop",
+					      NULL);
+
 	panel_place_menu_item_append_volumes (places_menu, FALSE);
 	add_menu_separator (places_menu);
 
 	panel_menu_items_append_from_desktop (places_menu,
-					      "network-scheme.desktop");
+					      "network-scheme.desktop",
+					      NULL);
 	panel_place_menu_item_append_volumes (places_menu, TRUE);
 
 	if (panel_is_program_in_path ("nautilus-connect-server")) {
@@ -514,7 +597,8 @@ panel_place_menu_item_create_menu (EggRecentViewGtk **recent_view)
 	add_menu_separator (places_menu);
 
 	panel_menu_items_append_from_desktop (places_menu,
-					      "gnome-search-tool.desktop");
+					      "gnome-search-tool.desktop",
+					      NULL);
 
 	*recent_view = panel_recent_append_documents_menu (places_menu,
 							   *recent_view);
@@ -541,10 +625,10 @@ panel_place_menu_item_recreate_menu (GtkWidget *widget)
 }
 
 static void
-panel_place_menu_item_is_home_dir_changed (GConfClient *client,
-					   guint        cnxn_id,
-					   GConfEntry  *entry,
-					   GtkWidget   *place_item)
+panel_place_menu_item_key_changed (GConfClient *client,
+				   guint        cnxn_id,
+				   GConfEntry  *entry,
+				   GtkWidget   *place_item)
 {
 	panel_place_menu_item_recreate_menu (place_item);
 }
@@ -598,8 +682,8 @@ panel_desktop_menu_item_append_menu (GtkWidget            *menu,
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 
-	panel_menu_items_append_from_desktop (menu, "yelp.desktop");
-	panel_menu_items_append_from_desktop (menu, "gnome-about.desktop");
+	panel_menu_items_append_from_desktop (menu, "yelp.desktop", NULL);
+	panel_menu_items_append_from_desktop (menu, "gnome-about.desktop", NULL);
 
 	if (parent->priv->append_lock_logout)
 		panel_menu_items_append_lock_logout (menu);
@@ -641,6 +725,9 @@ panel_place_menu_item_finalize (GObject *object)
 	gconf_client_remove_dir (panel_gconf_get_client (),
 				 DESKTOP_IS_HOME_DIR_DIR,
 				 NULL);
+	gconf_client_remove_dir (panel_gconf_get_client (),
+				 NAMES_DIR,
+				 NULL);
 
 	if (menuitem->priv->recent_view != NULL)
 		g_object_unref (menuitem->priv->recent_view);
@@ -687,9 +774,19 @@ panel_place_menu_item_instance_init (PanelPlaceMenuItem      *menuitem,
 			      DESKTOP_IS_HOME_DIR_DIR,
 			      GCONF_CLIENT_PRELOAD_NONE,
 			      NULL);
+	gconf_client_add_dir (panel_gconf_get_client (),
+			      NAMES_DIR,
+			      GCONF_CLIENT_PRELOAD_NONE,
+			      NULL);
 
+	panel_gconf_notify_add_while_alive (HOME_NAME_KEY,
+					    (GConfClientNotifyFunc) panel_place_menu_item_key_changed,
+					    G_OBJECT (menuitem));
 	panel_gconf_notify_add_while_alive (DESKTOP_IS_HOME_DIR_KEY,
-					    (GConfClientNotifyFunc) panel_place_menu_item_is_home_dir_changed,
+					    (GConfClientNotifyFunc) panel_place_menu_item_key_changed,
+					    G_OBJECT (menuitem));
+	panel_gconf_notify_add_while_alive (COMPUTER_NAME_KEY,
+					    (GConfClientNotifyFunc) panel_place_menu_item_key_changed,
 					    G_OBJECT (menuitem));
 
 	bookmarks_filename = g_build_filename (g_get_home_dir (),
@@ -698,12 +795,16 @@ panel_place_menu_item_instance_init (PanelPlaceMenuItem      *menuitem,
 
 	if (bookmarks_uri) {
 		GnomeVFSResult result;
+
 		result = gnome_vfs_monitor_add (&menuitem->priv->bookmarks_monitor,
 						bookmarks_uri,
 						GNOME_VFS_MONITOR_FILE,
 						panel_place_menu_item_gtk_bookmarks_changed,
 						menuitem);
-		if (result != GNOME_VFS_OK)
+
+		if (result == GNOME_VFS_ERROR_NOT_SUPPORTED)
+			g_message ("File monitoring not supported in the compiled version of gnome-vfs: bookmarks won't be monitored.");
+		else if (result != GNOME_VFS_OK)
 			g_warning ("Failed to add file monitor for %s: %s\n",
 				   bookmarks_uri,
 				   gnome_vfs_result_to_string (result));
@@ -960,7 +1061,7 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 		separator_inserted = GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
 	}
 
-	if (panel_is_program_in_path ("xscreensaver")) {
+	if (panel_lock_screen_action_available ("lock")) {
 		item = panel_menu_items_create_action_item (PANEL_ACTION_LOCK);
 		if (item != NULL) {
 			if (!separator_inserted) {
@@ -979,6 +1080,14 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
+
+	item = panel_menu_items_create_action_item (PANEL_ACTION_SHUTDOWN);
+	if (item != NULL) {
+		if (!separator_inserted)
+			add_menu_separator (menu);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
 }
 
 void
@@ -998,7 +1107,7 @@ panel_menu_item_activate_desktop_file (GtkWidget  *menuitem,
 		if (error) {
 			panel_error_dialog (menuitem_to_screen (menuitem),
 					    "cannot_launch_entry", TRUE,
-					    _("Cannot launch entry"),
+					    _("Could not launch menu item"),
 					    "%s",
 					    error->message);
 
@@ -1010,7 +1119,7 @@ panel_menu_item_activate_desktop_file (GtkWidget  *menuitem,
 
 		panel_error_dialog (menuitem_to_screen (menuitem),
 				    "cannot_load_entry", TRUE,
-				    _("Cannot load entry"),
+				    _("Could not load menu item"),
 				    "%s",
 				    error->message);
 		g_error_free (error);
