@@ -71,31 +71,36 @@ static GConfEnumStringPair panel_orientation_map [] = {
 	{ PANEL_ORIENTATION_TOP,    "top"    },
 	{ PANEL_ORIENTATION_BOTTOM, "bottom" },
 	{ PANEL_ORIENTATION_LEFT,   "left"   },
-	{ PANEL_ORIENTATION_RIGHT,  "right"  }
+	{ PANEL_ORIENTATION_RIGHT,  "right"  },
+	{ 0,                        NULL     }
 };
 
 static GConfEnumStringPair panel_animation_speed_map [] = {
 	{ PANEL_ANIMATION_SLOW,   "slow"   },
 	{ PANEL_ANIMATION_MEDIUM, "medium" },
-	{ PANEL_ANIMATION_FAST,   "fast"   }
+	{ PANEL_ANIMATION_FAST,   "fast"   },
+	{ 0,                      NULL     }
 };
 
 static GConfEnumStringPair panel_background_type_map [] = {
 	{ PANEL_BACK_NONE,  "gtk"   },
 	{ PANEL_BACK_COLOR, "color" },
-	{ PANEL_BACK_IMAGE, "image" }
+	{ PANEL_BACK_IMAGE, "image" },
+	{ 0,                NULL    }
 };
 
 static GConfEnumStringPair panel_object_type_map [] = {
-	{ PANEL_OBJECT_DRAWER,   "drawer-object" },
-	{ PANEL_OBJECT_MENU,     "menu-object" },
-	{ PANEL_OBJECT_LAUNCHER, "launcher-object" },
-	{ PANEL_OBJECT_BONOBO,   "bonobo-applet" },
-	{ PANEL_OBJECT_ACTION,   "action-applet" },
-	{ PANEL_OBJECT_MENU_BAR, "menu-bar" },
+	{ PANEL_OBJECT_DRAWER,    "drawer-object" },
+	{ PANEL_OBJECT_MENU,      "menu-object" },
+	{ PANEL_OBJECT_LAUNCHER,  "launcher-object" },
+	{ PANEL_OBJECT_BONOBO,    "bonobo-applet" },
+	{ PANEL_OBJECT_ACTION,    "action-applet" },
+	{ PANEL_OBJECT_MENU_BAR,  "menu-bar" },
+	{ PANEL_OBJECT_SEPARATOR, "separator" },
 	/* The following two are for backwards compatibility with 2.0.x */
-	{ PANEL_OBJECT_LOCK,     "lock-object" },
-	{ PANEL_OBJECT_LOGOUT,   "logout-object" }
+	{ PANEL_OBJECT_LOCK,      "lock-object" },
+	{ PANEL_OBJECT_LOGOUT,    "logout-object" },
+	{ 0,                      NULL }
 };
 
 static GQuark toplevel_id_quark = 0;
@@ -305,16 +310,30 @@ panel_profile_get_queued_changes (GObject *object)
 	return g_object_get_qdata (object, queued_changes_quark);
 }
 
+static void
+panel_profile_remove_commit_timeout (guint timeout)
+{
+	g_source_remove (timeout);
+}
+
 static void
 panel_profile_set_commit_timeout (PanelToplevel *toplevel,
 				  guint          timeout)
 {
+	GDestroyNotify destroy_notify;
+
 	if (!commit_timeout_quark)
 		commit_timeout_quark = g_quark_from_static_string ("panel-queued-timeout");
 
-	g_object_set_qdata (G_OBJECT (toplevel),
-			    commit_timeout_quark,
-			    GUINT_TO_POINTER (timeout));
+	if (timeout)
+		destroy_notify = (GDestroyNotify) panel_profile_remove_commit_timeout;
+	else
+		destroy_notify = NULL;
+
+	g_object_set_qdata_full (G_OBJECT (toplevel),
+				 commit_timeout_quark,
+				 GUINT_TO_POINTER (timeout),
+				 destroy_notify);
 }
 
 static guint
@@ -902,36 +921,38 @@ panel_profile_queue_toplevel_location_change (PanelToplevel          *toplevel,
 			panel_profile_get_toplevel_key (toplevel, "size"),
 			change->size);
 
-	if (change->x_changed)
-		gconf_change_set_set_int (
-			queued_changes,
-			panel_profile_get_toplevel_key (toplevel, "x"),
-			change->x);
-
-	if (change->x_centered_changed)
-		gconf_change_set_set_bool (
-			queued_changes,
-			panel_profile_get_toplevel_key (toplevel, "x_centered"),
-			change->x_centered);
-
-	if (change->y_changed)
-		gconf_change_set_set_int (
-			queued_changes,
-			panel_profile_get_toplevel_key (toplevel, "y"),
-			change->y);
-
-	if (change->y_centered_changed)
-		gconf_change_set_set_bool (
-			queued_changes,
-			panel_profile_get_toplevel_key (toplevel, "y_centered"),
-			change->y_centered);
-
 	if (change->orientation_changed)
 		gconf_change_set_set_string (
 			queued_changes,
 			panel_profile_get_toplevel_key (toplevel, "orientation"),
 			gconf_enum_to_string (panel_orientation_map, change->orientation));
 
+	if (!panel_toplevel_get_expand (toplevel)) {
+		if (change->x_changed)
+			gconf_change_set_set_int (
+				queued_changes,
+				panel_profile_get_toplevel_key (toplevel, "x"),
+				change->x);
+
+		if (change->x_centered_changed)
+			gconf_change_set_set_bool (
+				queued_changes,
+				panel_profile_get_toplevel_key (toplevel, "x_centered"),
+				change->x_centered);
+
+		if (change->y_changed)
+			gconf_change_set_set_int (
+				queued_changes,
+				panel_profile_get_toplevel_key (toplevel, "y"),
+				change->y);
+
+		if (change->y_centered_changed)
+			gconf_change_set_set_bool (
+				queued_changes,
+				panel_profile_get_toplevel_key (toplevel, "y_centered"),
+				change->y_centered);
+	}
+
 	commit_timeout = panel_profile_get_commit_timeout (G_OBJECT (toplevel));
 	if (!commit_timeout) {
 		commit_timeout =
@@ -1152,9 +1173,6 @@ panel_profile_background_change_notify (GConfClient   *client,
 	if (!strcmp (key, "type")) {
 		if (value->type == GCONF_VALUE_STRING) {
 			PanelBackgroundType  background_type;
-			const char          *str;
-
-			str = gconf_value_get_string (value);
 
 			if (panel_profile_map_background_type_string (
 						gconf_value_get_string (value),
@@ -1480,11 +1498,13 @@ panel_profile_delete_toplevel_objects (const char        *toplevel_id,
 
 		if (parent_toplevel_id && !strcmp (toplevel_id, parent_toplevel_id)) {
 			g_free (id);
+			g_free (parent_toplevel_id);
 			continue;
 		}
 
 		new_list = g_slist_prepend (new_list, id);
 
+		g_free (parent_toplevel_id);
 	}
 	g_slist_free (list);
 
@@ -1816,8 +1836,8 @@ panel_profile_load_object (GConfClient       *client,
         
 	if (!panel_profile_map_object_type_string (type_string, &object_type)) {
 		g_free (type_string);
-		g_free (object_dir);
 		gconf_client_remove_dir (client, object_dir, NULL);
+		g_free (object_dir);
 		return;
 	}
 	
@@ -2153,25 +2173,22 @@ panel_profile_copy_defaults_for_screen (GConfClient       *client,
 	GSList     *default_ids, *l;
 	GSList     *new_ids = NULL;
 	const char *key;
-	const char *id_list, *type_str, *schemas_str;
+	const char *id_list, *type_str;
 
 	id_list = panel_gconf_key_type_to_id_list (type);
 
 	switch (type) {
 	case PANEL_GCONF_TOPLEVELS:
 		type_str    = "toplevels";
-		schemas_str = "toplevels";
 		break;
 	case PANEL_GCONF_OBJECTS:
 		type_str    = "objects";
-		schemas_str = "objects";
 		break;
 	case PANEL_GCONF_APPLETS:
 		type_str    = "applets";
-		schemas_str = "objects";
 		break;
 	default:
-		type_str = schemas_str = NULL;
+		type_str = NULL;
 		g_assert_not_reached ();
 		break;
 	}
@@ -2418,9 +2435,9 @@ panel_profile_get_enable_program_list (void)
 }
 
 gboolean
-panel_profile_get_enable_autocomplete (void)
+panel_profile_get_enable_autocompletion (void)
 {
-	return get_program_listing_setting ("enable_autocomplete");
+	return get_program_listing_setting ("enable_autocompletion");
 }
 
 void
